//------------------------------------------------------------
/// \file	Main.cpp
/// \author	Rob Bateman
/// \date	9-feb-2005
/// \brief	This example demonstrates how to manipulate the texture
///			matrix stack. This allows us to do things such as 
///			translating the texture, scaling the texture etc. 
///				This simply creates a scrolling texture, and a 
///			rotating texture. 
//------------------------------------------------------------

#include <stdlib.h>
#include <iostream>
#include <GL/glut.h>
#include "texture.h"

/// this will hold the openGL texture object for the loaded texture
unsigned int g_Texture=0;

/// the position of the object
float scrollx = 0;
float rotate = 0;


//------------------------------------------------------------	OnDraw()
//
void OnDraw() {

	glClear(GL_DEPTH_BUFFER_BIT|GL_COLOR_BUFFER_BIT);

	// clear previous transformations
	glLoadIdentity();

	// set the camera position
	gluLookAt(0,0,10,0,0,0,0,1,0);

	// bind the environment map texture
	glBindTexture(GL_TEXTURE_2D,g_Texture);

	glPushMatrix();
		// move to the correct position
		glTranslatef(1.5,0,0);

		// switch to the texture matrix stack. This allows us to manipulate
		// the placement of textures.
		//
		glMatrixMode(GL_TEXTURE);

			// clear any previous transforms
			glLoadIdentity();

			// translate the texture a bit
			glTranslatef(scrollx,0,0);

		// revert to the modelview matrix to draw 3D geometry
		glMatrixMode(GL_MODELVIEW);
		
		// draw the teapot
		glutSolidTeapot(1);

	glPopMatrix();

	glPushMatrix();

		// move to the correct position
		glTranslatef(-1.5f,0.0f,0.0f);


		// switch to the texture matrix stack. This allows us to manipulate
		// the placement of textures.
		//
		glMatrixMode(GL_TEXTURE);

			// clear previous transforms
			glLoadIdentity();

			// rotate the texture a bit
			glRotatef(rotate,0.0f,0.0f,1.0f);

		// revert to the modelview matrix to draw 3D geometry
		glMatrixMode(GL_MODELVIEW);
		
		// draw the teapot
		glutSolidTeapot(1);

	glPopMatrix();

	glutSwapBuffers();
}

//------------------------------------------------------------	OnInit()
//
void OnInit() {

	// enable depthe testing
	glEnable(GL_DEPTH_TEST);

	// enable texturing
	glEnable(GL_TEXTURE_2D);

	// tell openGL to generate the texture coords for a sphere map
	glTexGeni(GL_S,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);
	glTexGeni(GL_T,GL_TEXTURE_GEN_MODE,GL_EYE_LINEAR);

	// enable automatic texture coordinate generation
	glEnable(GL_TEXTURE_GEN_S);
	glEnable(GL_TEXTURE_GEN_T);

	// enable a simple light
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// load texture as compressed
	g_Texture = LoadTexture("texture.bmp");

	// print some info about it
	std::cout << "Texture Size=" << (GetTextureSize(g_Texture)/1024.0f) << "Kb"<< std::endl;
}

//------------------------------------------------------------	OnReshape()
//
void OnReshape(int w, int h)
{
	// prevent division by zero when minimised
	if (h==0) 
		h=1;
	glViewport(0,0,w,h);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(45,(float)w/h,0.1,100);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}


//------------------------------------------------------------	OnExit()
//
void OnExit() {
	DeleteAllTextures();
}

//------------------------------------------------------------	OnIdle()
//
void OnIdle() {
	
	// update the texture transformation variables
	scrollx += 0.01f;
	rotate += 0.5f;

	// redraw the screen
	glutPostRedisplay();
}


//------------------------------------------------------------	main()
//
int main(int argc,char** argv) {

	glutInit(&argc,argv);
	glutInitDisplayMode(GLUT_DEPTH|GLUT_RGBA|GLUT_DOUBLE);
	glutInitWindowSize(640,480);
	glutCreateWindow("Texture Matrix Stack Example");

	glutDisplayFunc(OnDraw);
	glutReshapeFunc(OnReshape);
	glutIdleFunc(OnIdle);
	
	OnInit();

	atexit(OnExit);

	glutMainLoop();
	return 0;
}
